// Ovladani RGB WS2812 LED diody pro signalizacni ucely
/**The MIT License (MIT)
Copyright (c) 2017,2020 by Pavel Brychta
Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:
The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
See more at http://www.xpablo.cz
*/

#ifndef __WSLED_H__
#define __WSLED_H__
#include <Arduino.h>
#include <Ticker.h>

struct LEDRGB {
    union {
        struct {
            union {
                uint8_t r;
                uint8_t red;
            };
            union {
                uint8_t g;
                uint8_t green;
            };
            union {
                uint8_t b;
                uint8_t blue;
            };
        };
        uint8_t raw[3];
    };

    // Array access operator to index into the crgb object
    inline uint8_t& operator[] (uint8_t x) __attribute__((always_inline))
    {
        return raw[x];
    }

    // Array access operator to index into the crgb object
    inline const uint8_t& operator[] (uint8_t x) const __attribute__((always_inline))
    {
        return raw[x];
    }

    // default values are UNINITIALIZED
    inline LEDRGB() __attribute__((always_inline))
    {
    }

    // allow construction from R, G, B
    inline LEDRGB( uint8_t ir, uint8_t ig, uint8_t ib)  __attribute__((always_inline))
        : r(ir), g(ig), b(ib)
    {
    }

    // allow construction from 32-bit (really 24-bit) bit 0xRRGGBB color code
    inline LEDRGB( uint32_t colorcode)  __attribute__((always_inline))
        : r((colorcode >> 16) & 0xFF), g((colorcode >> 8) & 0xFF), b((colorcode >> 0) & 0xFF)
    {
    }

    // allow copy construction
    inline LEDRGB(const LEDRGB& rhs) __attribute__((always_inline))
    {
        r = rhs.r;
        g = rhs.g;
        b = rhs.b;
    }

    // allow assignment from one RGB struct to another
    inline LEDRGB& operator= (const LEDRGB& rhs) __attribute__((always_inline))
    {
        r = rhs.r;
        g = rhs.g;
        b = rhs.b;
        return *this;
    }

    // allow assignment from 32-bit (really 24-bit) 0xRRGGBB color code
    inline LEDRGB& operator= (const uint32_t colorcode) __attribute__((always_inline))
    {
        r = (colorcode >> 16) & 0xFF;
        g = (colorcode >>  8) & 0xFF;
        b = (colorcode >>  0) & 0xFF;
        return *this;
    }

    // allow assignment from R, G, and B
    inline LEDRGB& setRGB (uint8_t nr, uint8_t ng, uint8_t nb) __attribute__((always_inline))
    {
        r = nr;
        g = ng;
        b = nb;
        return *this;
    }
};

enum ColorOrder {
    RGB = 0012,
    GRB = 0102
};

typedef enum {
    LED_STOP = 0,
    LED_BLINK,
    LED_BREATH,
    LED_PULSE,
    LED_PATTERN,
}ledstate_t;

class wsLED
{
protected:
    int _pin; // pin, na kterem je LED pripojena
    LEDRGB _color1;
    LEDRGB _color2;
    int _order; // ColorOrder
    uint32_t _endtime;
    uint8_t _scale;
    uint8_t _color[3];
    Ticker _handler;
    ledstate_t _ledState; // stav LEDky (blika, dycha, ...)
    ledstate_t _savedState; // stav pro navrat
    int _subState; // pomocny stav/index v poli dychani
#if defined(ARDUINO_ARCH_ESP32)
    int _rmtChannel;
#endif

    bool canShow(void);
    void show(void);
    void setColor(LEDRGB &color);
#if defined(ARDUINO_ARCH_ESP32)
    portMUX_TYPE _mux = portMUX_INITIALIZER_UNLOCKED;
#endif

public:
    wsLED(int pin, int order = RGB);
    wsLED(void);
    void begin(void);
    void begin(int pin);
    static void lh(wsLED *ptr);
    void rtLed(void); // vykonna metoda
    void setColors(LEDRGB color1, LEDRGB color2 = LEDRGB(0, 0, 0));
    void blink(int speed = 300);
    void breath(void);
    void setOrder(int neworder);
    ledstate_t getState(void);
    void pulse(LEDRGB color, uint32_t duration);
};

#endif

